==> helpdesk.ron.cz?menuid=download&filename=ronportal_4_2107_155.zip: ronportal/ronportal/editace/podlekalendareformular.php <==
<?php
/**
 * stanislav.elbl, 26.02.2021
 */

namespace RonPortal\Editace;

use Exception;
use Ron\Controls\ComboBox;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\DB\SQLUtils;
use Ron\MessageManager;
use Ron\MVC\ViewObject;
use Ron\RonException;
use Ron\TextManager;
use RonPortal\Dochazka\DochazkaUtils;
use RonPortal\NeniPravoException;
use RonPortal\Obdobi;
use RonPortal\PracovniPomer;
use RonPortal\Prava;
use RonPortal\RonPortalSession;
use RonPortal\RonPortalSettings;
use RonPortal\System\AgentManager;

// TODO: Predelat na InputForm pro moznost konfigurace

class PodleKalendareFormular implements ViewObject
{
    /** @var  Database */
    protected $database;

    /** @var PracovniPomer */
    public $pp;
    /** @var Obdobi */
    public $obdobi;

//    var $f_Schvaleni; // schvaleni dochazky
    protected $datumZpracovaniDochazky;

    public $datumOd;
    public $datumDo;
    public $datumOdStr;
    public $datumDoStr;

    protected $operace;
    protected $rozdeleni;
    /** @var int     cela smena, 1./2. pulka */
    protected $delka = 0;

    protected $pouzitPocatek; // pouzit pocatek a konec smeny
    protected $pevneJadro;    // pouzit pevne jadro

    protected $automatickyOpakovat;
    protected $vlozitPrestavky;


    public function __construct($database)
    {
        $this ->database = $database;
    }


    /**
     * @throws NeniPravoException
     * @throws Exception
     */
    public function nactiDataPodlePost()
    {
        $OK = true;

        $this ->datumOdStr = $_POST['DatumOd'];
        $this ->datumDoStr = $_POST['DatumDo'];
        $this ->operace = $_POST['operace'];
        $this ->delka = (int)$_POST['delka'];

        // zaskrtavatka
        $this ->pouzitPocatek = isset($_POST['pocatekkonec']) ? (int)$_POST['pocatekkonec'] : 0; // zaskrtavatko pouzit pocatek a konec smeny
        $this ->pevneJadro = isset($_POST['pevnejadro']) ? (int)$_POST['pevnejadro'] : 0; // zaskrtavatko pouzit pevne jadro
        $this ->automatickyOpakovat = isset($_POST['autoopakovat']) ? (int)$_POST['autoopakovat'] : 0;
        $this ->vlozitPrestavky =
            RonPortalSettings::getIntValue(RonPortalSettings::EDITACE_VKLADAT_PRESTAVKY)
            ||  (isset($_POST['vlozitprestavky']) ? (int)$_POST['vlozitprestavky'] : 0);

        if (RonPortalSettings::getIntValue(RonPortalSettings::EDITACE_TYP_ROZDELENI)) {
            $this ->rozdeleni = $_POST['rozdeleni_popis'];
        }

        $this ->obdobi ->nactiObdobi($this ->database, 'ADS', $_POST['obdobi']);

        if (!isset($_POST['DatumOd']) || $_POST['DatumOd'] === '') {
            throw new RonException( TextManager::_('form.Missing_value') . ' ' . TextManager::_('Datum_od') . '!', 'DatumOd');
        }

        $this ->datumOd = DateUtils::dateStringToTimestamp($_POST['DatumOd'], 'DatumOd');

        if (!isset($_POST['DatumDo']) || $_POST['DatumDo'] === '') {
            throw new RonException( TextManager::_('form.Missing_value') . ' ' . TextManager::_('Datum_do') . '!', 'DatumDo');
        }

        $this ->datumDo = DateUtils::dateStringToTimestamp($_POST['DatumDo'], 'DatumDo');

        // datumy jsou spravne zadane - jeste zkontroluje obdobi atd.
        if ($OK  &&  $this ->datumDo < $this ->datumOd) {
            throw new RonException( TextManager::_('Pocatecni_datum_vetsi_nez_koncove'), 'DatumDo' );
        }

        $mindatum = $this ->obdobi ->zacatek;
        $maxdatum = $this ->obdobi ->konec;
        if ($maxdatum > $this ->datumZpracovaniDochazky) {
            $maxdatum = $this ->datumZpracovaniDochazky;
        }
        if ($maxdatum < $mindatum) {
            throw new Exception(TextManager::_('Dochazka_jeste_neni_zpracovana'));
        }

        if ($OK) {
            if ($this->datumOd < $mindatum || $this->datumDo > $maxdatum) {
                throw new Exception(sprintf(TextManager::_('Datum_musi_byt_v_rozsahu'), date('j.n.Y', $mindatum), date('j.n.Y', $maxdatum)));
            }
        }

        if (!isset($this ->operace) || $this ->operace === '') {
            throw new RonException( TextManager::_('form.Missing_value') . ' ' . TextManager::_('Operace'), 'operace' );
        }

        if (!EditaceUtils::jePravoEditovatOperaci($this ->operace)) {
            throw new NeniPravoException();
        }

        if (!isset($this ->delka)) {
            throw new Exception( TextManager::_('form.Missing_value') . ' ' . TextManager::_('Delka') );
        }

        if (!$this ->rozdeleni) {
            if (EditaceUtils::jeRozdeleniDochazkyPovinne($this->database, $this ->operace)) {
                throw new RonException(
                    TextManager::_('Neni_zadano_rozdeleni_dochazky') .
                    (RonPortalSettings::getValue(RonPortalSettings::EDITACE_NADPIS_ROZDELENI)
                        ? ' (' . RonPortalSettings::getValue(RonPortalSettings::EDITACE_NADPIS_ROZDELENI) . ')'
                        : ''),
                    'rozdeleni_popis'
                );
            }
        }
    }


    /**
     * @throws DBException
     * @throws Exception
     */
    public function nactiData($idpp, $den)
    {
        $this ->pp = new PracovniPomer();
        if (!$this ->pp ->nactiPPPodleId($this ->database, $idpp)) {
            throw new Exception('Zadany pracovni pomer nebyl nalezen!');
        }

        $this ->obdobi = new Obdobi();

        $this ->datumZpracovaniDochazky = DochazkaUtils::datumCasZpracovaniDochazky($this ->database);

        // $this -> f_Schvaleni = new SchvaleniDochazky();

        if ($den) {
            // formular je volan pro konkretni datum (napr. z editace dne) - predvyplni zadany den
            $this ->datumOd = $den;
            $this ->datumDo = $den;

            $this ->obdobi ->nactiObdobiProDen($this ->database, 'ADS', $den);
        } else {
            $this ->obdobi ->nactiObdobi($this ->database, 'ADS', RonPortalSession::obdobi());
        }

        if ($this ->datumOd) {
            $this ->datumOdStr = date('j.n.Y', $this ->datumOd);
        }

        if ($this ->datumDo) {
            $this ->datumDoStr = date('j.n.Y', $this ->datumDo);
        }
    }


    /**
     * @throws DBException
     */
    protected function renderVyberOperace($popis)
    {
        $params = $this ->database ->createParameters();
        $sql =
            'SELECT Kod, Popis FROM OperaceDochazky ' .
            ' WHERE VkladatVEditaciDochazky = 1';

        $prava = Prava::vratPrava('editace');
        if (trim($prava ->priznak3) !== '') {
            $sql .= ' AND ' . SQLUtils::getINCondition($this ->database, 'Kod', explode(';', $prava ->priznak3), $params);
        }

        $sql .=
            ' ORDER BY ' . ($popis ? 'Popis' : 'Kod');

        $combo = new ComboBox($popis ? 'operace' : 'operace_kod');
        $combo ->captionIndex = $popis ? 1 : 0;
        $combo ->emptyRow = ' ';
        $combo ->className = 'form-input ' . MessageManager::getClass('operace') ;
        $combo ->loadData($this ->database, $sql, $this ->operace, $params);
        $combo ->htmlParams = [
            'data-ron-select2' => $popis ? 'operace_kod' : 'operace'
        ];
        $combo ->render();
    }


    protected function renderVyberRozdeleni($popis)
    {
        $params = $this ->database ->createParameters();
        $sql =
            'SELECT Kod, Popis FROM Rozdeleni ' .
            ' WHERE KodTypuRozdeleni = ' . $params ->addParameter(RonPortalSettings::getIntValue(RonPortalSettings::EDITACE_TYP_ROZDELENI)) .
            '   AND (DatumZruseni >= ' . $params ->addParameter($this ->database ->dateToParam($this ->datumDo)) . ' OR DatumZruseni IS NULL) ' .
            '   AND DatumVytvoreni <= ' . $params ->addParameter($this ->database ->dateToParam($this ->datumOd));

        $sql .= ' ORDER BY '.($popis ? 'Popis' : 'Kod');

        $combo = new ComboBox($popis ? 'rozdeleni_popis' : 'rozdeleni_kod');
        $combo ->alwaysShowSelected = true;
        $combo ->captionIndex = $popis ? 1 : 0;
        $combo ->emptyRow = ' ';
        $combo ->className = 'form-input ' . MessageManager::getClass('rozdeleni_popis');
        $combo ->loadData($this ->database, $sql, $this ->rozdeleni, $params);
        $combo ->htmlParams = [
            'data-ron-select2' => $popis ? 'rozdeleni_kod' : 'rozdeleni_popis'
        ];
        $combo ->render();
    }


    public function render()
    {
        require 'ronportal/editace/view_podle_kalendare.php';
    }


    /**
     * @throws Exception
     */
    public function provedVlozeni()
    {
        if ( RonPortalSettings::getSpousteniADS() ) {
            $params = '9 ' .
                ' -interval:' . date('Y-n-j', $this ->datumOd) . ';' . date('Y-n-j', $this ->datumDo) .
                ' -IDPP:' . $this ->pp ->idPP .
                ' -operace:' . $this ->operace . ' -delka:' . $this ->delka .
                ' -alwaysLog ';

            if ( RonPortalSettings::getIntValue(RonPortalSettings::EDITACE_TYP_ROZDELENI)  &&  $this ->rozdeleni != '' ) {
                $params .= ' -attSubDiv:"' . RonPortalSettings::getIntValue(RonPortalSettings::EDITACE_TYP_ROZDELENI) . '/' . $this ->rozdeleni . '"';
            }

            $params .=
                ($this ->pouzitPocatek ? ' -pouzitKonecSmeny' : '') .
                ($this ->pevneJadro ? ' -pevneJadro' : '') .
                ($this ->automatickyOpakovat ? ' -autoOpakovat' : '') .
                ($this ->vlozitPrestavky ? ' -vkladatPrestavky' : '');

            return AgentManager::executeADS('ADS_Agent_Processing.exe', $params);
        }

        return false;
    }

}

==> helpdesk.ron.cz?menuid=download&filename=ronportal_4_2206_162.zip: ronportal/ronportal/editace/podlekalendareformular.php <==
<?php
/**
 * stanislav.elbl, 26.02.2021
 */

namespace RonPortal\Editace;

use Exception;
use Ron\Controls\ComboBox;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\DB\SQLUtils;
use Ron\MessageManager;
use Ron\MVC\ViewObject;
use Ron\RonException;
use Ron\TextManager;
use RonPortal\Dochazka\DochazkaUtils;
use RonPortal\NeniPravoException;
use RonPortal\Obdobi;
use RonPortal\PracovniPomer;
use RonPortal\Prava;
use RonPortal\RonPortalSession;
use RonPortal\RonPortalSettings;
use RonPortal\System\AgentManager;

// TODO: Predelat na InputForm pro moznost konfigurace

class PodleKalendareFormular implements ViewObject
{
    /** @var  Database */
    protected $database;

    /** @var PracovniPomer */
    public $pp;
    /** @var Obdobi */
    public $obdobi;

//    var $f_Schvaleni; // schvaleni dochazky
    protected $datumZpracovaniDochazky;

    public $datumOd;
    public $datumDo;
    public $datumOdStr;
    public $datumDoStr;

    protected $operace;
    protected $rozdeleni;
    /** @var int     cela smena, 1./2. pulka */
    protected $delka = 0;

    protected $pouzitPocatek; // pouzit pocatek a konec smeny
    protected $pevneJadro;    // pouzit pevne jadro

    protected $automatickyOpakovat;
    protected $vlozitPrestavky;


    public function __construct($database)
    {
        $this ->database = $database;
    }


    /**
     * @throws NeniPravoException
     * @throws Exception
     */
    public function nactiDataPodlePost()
    {
        $OK = true;

        $this ->datumOdStr = $_POST['DatumOd'];
        $this ->datumDoStr = $_POST['DatumDo'];
        $this ->operace = $_POST['operace'];
        $this ->delka = (int)$_POST['delka'];

        // zaskrtavatka
        $this ->pouzitPocatek = isset($_POST['pocatekkonec']) ? (int)$_POST['pocatekkonec'] : 0; // zaskrtavatko pouzit pocatek a konec smeny
        $this ->pevneJadro = isset($_POST['pevnejadro']) ? (int)$_POST['pevnejadro'] : 0; // zaskrtavatko pouzit pevne jadro
        $this ->automatickyOpakovat = isset($_POST['autoopakovat']) ? (int)$_POST['autoopakovat'] : 0;
        $this ->vlozitPrestavky =
            RonPortalSettings::getIntValue(RonPortalSettings::EDITACE_VKLADAT_PRESTAVKY)
            ||  (isset($_POST['vlozitprestavky']) ? (int)$_POST['vlozitprestavky'] : 0);

        if (RonPortalSettings::getIntValue(RonPortalSettings::EDITACE_TYP_ROZDELENI)) {
            $this ->rozdeleni = $_POST['rozdeleni_popis'];
        }

        $this ->obdobi ->nactiObdobi($this ->database, 'ADS', $_POST['obdobi']);

        if (!isset($_POST['DatumOd']) || $_POST['DatumOd'] === '') {
            throw new RonException( TextManager::_('form.Missing_value') . ' ' . TextManager::_('Datum_od') . '!', 'DatumOd');
        }

        $this ->datumOd = DateUtils::dateStringToTimestamp($_POST['DatumOd'], 'DatumOd');

        if (!isset($_POST['DatumDo']) || $_POST['DatumDo'] === '') {
            throw new RonException( TextManager::_('form.Missing_value') . ' ' . TextManager::_('Datum_do') . '!', 'DatumDo');
        }

        $this ->datumDo = DateUtils::dateStringToTimestamp($_POST['DatumDo'], 'DatumDo');

        // datumy jsou spravne zadane - jeste zkontroluje obdobi atd.
        if ($OK  &&  $this ->datumDo < $this ->datumOd) {
            throw new RonException( TextManager::_('Pocatecni_datum_vetsi_nez_koncove'), 'DatumDo' );
        }

        $mindatum = $this ->obdobi ->zacatek;
        $maxdatum = $this ->obdobi ->konec;
        if ($maxdatum > $this ->datumZpracovaniDochazky) {
            $maxdatum = $this ->datumZpracovaniDochazky;
        }
        if ($maxdatum < $mindatum) {
            throw new Exception(TextManager::_('Dochazka_jeste_neni_zpracovana'));
        }

        if ($OK) {
            if ($this->datumOd < $mindatum || $this->datumDo > $maxdatum) {
                throw new Exception(sprintf(TextManager::_('Datum_musi_byt_v_rozsahu'), date('j.n.Y', $mindatum), date('j.n.Y', $maxdatum)));
            }
        }

        if (!isset($this ->operace) || $this ->operace === '') {
            throw new RonException( TextManager::_('form.Missing_value') . ' ' . TextManager::_('Operace'), 'operace' );
        }

        if (!EditaceUtils::jePravoEditovatOperaci($this ->operace)) {
            throw new NeniPravoException();
        }

        if (!isset($this ->delka)) {
            throw new Exception( TextManager::_('form.Missing_value') . ' ' . TextManager::_('Delka') );
        }

        if (!$this ->rozdeleni) {
            if (EditaceUtils::jeRozdeleniDochazkyPovinne($this->database, $this ->operace)) {
                throw new RonException(
                    TextManager::_('Neni_zadano_rozdeleni_dochazky') .
                    (RonPortalSettings::getValue(RonPortalSettings::EDITACE_NADPIS_ROZDELENI)
                        ? ' (' . RonPortalSettings::getValue(RonPortalSettings::EDITACE_NADPIS_ROZDELENI) . ')'
                        : ''),
                    'rozdeleni_popis'
                );
            }
        }
    }


    /**
     * Nacte data pro zobrazeni formulare podle zadanych parametru (pracovni pomer a prednastavene obdobi)
     * @param int $idPP     Id pracovniho pomeru
     * @param int $den      Prednastaveny datum
     * @param int $denDo    Prednastaveny datum do.
     * @throws Exception
     * @throws NeniPravoException
     */
    public function nactiData($idPP, $den = null, $denDo = null)
    {
        $this ->pp = new PracovniPomer();
        if (!$this ->pp ->nactiPPPodleId($this ->database, $idPP)) {
            throw new Exception('Zadany pracovni pomer nebyl nalezen!');
        }

        $this ->obdobi = new Obdobi();

        $this ->datumZpracovaniDochazky = DochazkaUtils::datumCasZpracovaniDochazky($this ->database);

        // $this -> f_Schvaleni = new SchvaleniDochazky();

        if ($den) {
            // formular je volan pro konkretni datum (napr. z editace dne) - predvyplni zadany den
            $this ->datumOd = $den;
            $this ->datumDo = $denDo ?: $den;

            $this ->obdobi ->nactiObdobiProDen($this ->database, 'ADS', $den);
        } else {
            $this ->obdobi ->nactiObdobi($this ->database, 'ADS', RonPortalSession::obdobi());
        }

        Prava::nastavObdobiProPrava($this ->obdobi ->zacatek, $this ->obdobi ->konec);
        if (!Prava::lzeZobrazitPP('editace', $this ->pp ->idPP)) {
            throw new NeniPravoException();
        }

        if ($this ->datumOd) {
            $this ->datumOdStr = date('j.n.Y', $this ->datumOd);
        }

        if ($this ->datumDo) {
            $this ->datumDoStr = date('j.n.Y', $this ->datumDo);
        }
    }


    /**
     * @throws DBException
     */
    protected function renderVyberOperace($popis)
    {
        $params = $this ->database ->createParameters();
        $sql =
            'SELECT Kod, Popis FROM OperaceDochazky ' .
            ' WHERE VkladatVEditaciDochazky = 1';

        $prava = Prava::vratPrava('editace');
        if (trim($prava ->priznak3) !== '') {
            $sql .= ' AND ' . SQLUtils::getINCondition($this ->database, 'Kod', explode(';', $prava ->priznak3), $params);
        }

        $sql .=
            ' ORDER BY ' . ($popis ? 'Popis' : 'Kod');

        $combo = new ComboBox($popis ? 'operace' : 'operace_kod');
        $combo ->captionIndex = $popis ? 1 : 0;
        $combo ->emptyRow = ' ';
        $combo ->className = 'form-input ' . MessageManager::getClass('operace') ;
        $combo ->loadData($this ->database, $sql, $this ->operace, $params);
        $combo ->htmlParams = [
            'data-ron-select2' => $popis ? 'operace_kod' : 'operace'
        ];
        $combo ->render();
    }


    protected function renderVyberRozdeleni($popis)
    {
        $params = $this ->database ->createParameters();
        $sql =
            'SELECT Kod, Popis FROM Rozdeleni ' .
            ' WHERE KodTypuRozdeleni = ' . $params ->addParameter(RonPortalSettings::getIntValue(RonPortalSettings::EDITACE_TYP_ROZDELENI)) .
            '   AND (DatumZruseni >= ' . $params ->addParameter($this ->database ->dateToParam($this ->datumDo)) . ' OR DatumZruseni IS NULL) ' .
            '   AND DatumVytvoreni <= ' . $params ->addParameter($this ->database ->dateToParam($this ->datumOd));

        $sql .= ' ORDER BY '.($popis ? 'Popis' : 'Kod');

        $combo = new ComboBox($popis ? 'rozdeleni_popis' : 'rozdeleni_kod');
        $combo ->alwaysShowSelected = true;
        $combo ->captionIndex = $popis ? 1 : 0;
        $combo ->emptyRow = ' ';
        $combo ->className = 'form-input ' . MessageManager::getClass('rozdeleni_popis');
        $combo ->loadData($this ->database, $sql, $this ->rozdeleni, $params);
        $combo ->htmlParams = [
            'data-ron-select2' => $popis ? 'rozdeleni_kod' : 'rozdeleni_popis'
        ];
        $combo ->render();
    }


    public function render()
    {
        require 'ronportal/editace/view_podle_kalendare.php';
    }


    /**
     * Provede vlozeni dochazky podle kalendare podle dat zadanych uzivatelem (datumy, operace atd.)
     * @throws Exception
     */
    public function provedVlozeni()
    {
        $prava = Prava::vratPrava('editace');
        if (!$prava ->lzeZobrazitPP($this ->pp ->idPP)) {
            throw new NeniPravoException();
        }

        if ( RonPortalSettings::getSpousteniADS() ) {
            $params = '9 ' .
                ' -interval:' . date('Y-n-j', $this ->datumOd) . ';' . date('Y-n-j', $this ->datumDo) .
                ' -IDPP:' . $this ->pp ->idPP .
                ' -operace:' . $this ->operace . ' -delka:' . $this ->delka .
                ' -alwaysLog ';

            if ( RonPortalSettings::getIntValue(RonPortalSettings::EDITACE_TYP_ROZDELENI)  &&  $this ->rozdeleni != '' ) {
                $params .= ' -attSubDiv:"' . RonPortalSettings::getIntValue(RonPortalSettings::EDITACE_TYP_ROZDELENI) . '/' . $this ->rozdeleni . '"';
            }

            $params .=
                ($this ->pouzitPocatek ? ' -pouzitKonecSmeny' : '') .
                ($this ->pevneJadro ? ' -pevneJadro' : '') .
                ($this ->automatickyOpakovat ? ' -autoOpakovat' : '') .
                ($this ->vlozitPrestavky ? ' -vkladatPrestavky' : '');

            AgentManager::executeADS('ADS_Agent_Processing.exe', $params);
        }
    }

}
